@@ -18,29 +18,23 @@
  */
 
 #include "ApplicationProxyModelHelper.h"
-#include <ApplicationModel/ApplicationModel.h>
-#include "BackendsSingleton.h"
+#include <resources/ResourcesModel.h>
 #include <QDebug>
-#include <LibQApt/Backend>
 
 ApplicationProxyModelHelper::ApplicationProxyModelHelper(QObject* parent)
-    : ApplicationProxyModel(parent)
+    : ResourcesProxyModel(parent)
 {
-    if(BackendsSingleton::self()->backend())
-        init();
-    else
-        connect(BackendsSingleton::self(), SIGNAL(initialized()), SLOT(init()));
+    init();
 }
 
 void ApplicationProxyModelHelper::init()
 {
-    setBackend(BackendsSingleton::self()->backend());
-    setSourceModel(BackendsSingleton::self()->appsModel());
+    setSourceModel(ResourcesModel::global());
     
     if(!m_sortRoleString.isEmpty())
         setStringSortRole_hack(m_sortRoleString);
     
-    connect(BackendsSingleton::self()->appsModel(), SIGNAL(modelReset()),
+    connect(ResourcesModel::global(), SIGNAL(modelReset()),
             SLOT(sortModel()));
     sortModel();
 }
@@ -50,17 +44,6 @@ void ApplicationProxyModelHelper::sortModel()
     QSortFilterProxyModel::sort(sortColumn(), sortOrder());
 }
 
-void ApplicationProxyModelHelper::setStateFilter_hack(int state)
-{
-    setStateFilter((QApt::Package::State) state);
-    emit stateFilterChanged();
-}
-
-Application* ApplicationProxyModelHelper::applicationAt(int row)
-{
-    return ApplicationProxyModel::applicationAt(index(row, 0));
-}
-
 int ApplicationProxyModelHelper::stringToRole(const QByteArray& strRole) const
 {
     return roleNames().key(strRole);
