@@ -23,12 +23,12 @@
 #include <KCmdLineArgs>
 #include <KUniqueApplication>
 #include <KStandardDirs>
-#include "MuonInstallerMainWindow.h"
+#include "MuonDiscoverMainWindow.h"
 
 static const char description[] =
     I18N_NOOP("An application discoverer");
 
-static const char version[] = "1.4.0";
+static const char version[] = "1.9.60";
 
 int main(int argc, char** argv)
 {
@@ -60,7 +60,7 @@ int main(int argc, char** argv)
     app.disableSessionManagement();
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
     
-    MuonInstallerMainWindow *mainWindow = new MuonInstallerMainWindow;
+    MuonDiscoverMainWindow *mainWindow = new MuonDiscoverMainWindow;
     if(args->isSet("application"))
         mainWindow->openApplication(args->getOption("application"));
     else if(args->isSet("mime"))
