@@ -18,7 +18,7 @@
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
-#include "ApplicationWindow.h"
+#include "MainWindow.h"
 
 #include <KAboutData>
 #include <KCmdLineArgs>
@@ -30,7 +30,7 @@
 static const char description[] =
     I18N_NOOP("An application manager");
 
-static const char version[] = "1.4.1";
+static const char version[] = "1.9.60";
 
 int main(int argc, char **argv)
 {
@@ -55,7 +55,7 @@ int main(int argc, char **argv)
     KGlobal::dirs()->addResourceDir("appicon", "/usr/share/app-install/icons/");
     app.disableSessionManagement();
 
-    ApplicationWindow *mainWindow = new ApplicationWindow;
+    MainWindow *mainWindow = new MainWindow;
     mainWindow->show();
 
     return app.exec();
