@@ -26,6 +26,7 @@
 #include <KUrl>
 
 #include "libmuonprivate_export.h"
+#include "AbstractReviewsBackend.h"
 
 namespace QOAuth {
     class Interface;
@@ -43,20 +44,18 @@ class Application;
 class Rating;
 class Review;
 
-class MUONPRIVATE_EXPORT ReviewsBackend : public QObject
+class MUONPRIVATE_EXPORT ReviewsBackend : public AbstractReviewsBackend
 {
     Q_OBJECT
-    Q_PROPERTY(bool hasCredentials READ hasCredentials NOTIFY loginStateChanged)
-    Q_PROPERTY(QString name READ userName NOTIFY loginStateChanged)
 public:
     ReviewsBackend(QObject *parent);
     ~ReviewsBackend();
 
-    Q_SCRIPTABLE Rating *ratingForApplication(Application *app) const;
+    Rating *ratingForApplication(AbstractResource *app) const;
 
     void setAptBackend(QApt::Backend *aptBackend);
-    void fetchReviews(Application* app, int page=1);
-    void clearReviewCache();
+    void fetchReviews(AbstractResource* app, int page=1);
+//     void clearReviewCache();
     void stopPendingJobs();
     bool isFetching() const;
 
@@ -92,16 +91,11 @@ public slots:
     void registerAndLogin();
     void logout();
     void submitUsefulness(Review* r, bool useful);
-    void submitReview(Application* app, const QString& summary,
+    void submitReview(AbstractResource* app, const QString& summary,
                       const QString& review_text, const QString& rating);
     void deleteReview(Review* r);
     void flagReview(Review* r, const QString& reason, const QString &text);
     void refreshConsumerKeys();
-
-Q_SIGNALS:
-    void reviewsReady(Application *app, QList<Review *>);
-    void ratingsReady();
-    void loginStateChanged();
 };
 
 #endif
