@@ -60,7 +60,7 @@
 
 bool packageNameLessThan(QApt::Package *p1, QApt::Package *p2)
 {
-     return p1->latin1Name() < p2->latin1Name();
+     return p1->name() < p2->name();
 }
 
 QApt::PackageList sortPackages(QApt::PackageList list)
@@ -148,10 +148,6 @@ PackageWidget::PackageWidget(QWidget *parent)
     splitter->addWidget(bottomVBox);
 }
 
-PackageWidget::~PackageWidget()
-{
-}
-
 void PackageWidget::setupActions()
 {
     m_installAction = new KAction(this);
@@ -233,6 +229,7 @@ void PackageWidget::setBackend(QApt::Backend *backend)
     m_proxyModel->setBackend(m_backend);
     m_packageView->setSortingEnabled(true);
     QApt::PackageList packageList = m_backend->availablePackages();
+    emit doneSortingPackages(false);
     QFuture<QList<QApt::Package*> > future = QtConcurrent::run(sortPackages, packageList);
     m_watcher->setFuture(future);
     m_packageView->updateView();
@@ -276,7 +273,7 @@ void PackageWidget::contextMenuRequested(const QPoint &pos)
     menu.addSeparator();
     menu.addAction(m_lockAction);
 
-    const QModelIndexList selected = m_packageView->currentSelection();
+    const QModelIndexList selected = m_packageView->selectionModel()->selectedIndexes();
 
     if (!selected.size()) {
         return;
@@ -346,6 +343,7 @@ void PackageWidget::setSortedPackages()
     m_searchEdit->setEnabled(true);
     m_busyWidget->stop();
     QApplication::restoreOverrideCursor();
+    emit doneSortingPackages(true);
 }
 
 void PackageWidget::sectionClicked(int section)
@@ -549,11 +547,6 @@ void PackageWidget::setPackagesKeep()
     actOnPackages(QApt::Package::ToKeep);
 }
 
-bool PackageWidget::setLocked(QApt::Package *package, bool lock)
-{
-   return m_backend->setPackagePinned(package, lock);
-}
-
 void PackageWidget::setPackagesLocked(bool lock)
 {
     const QApt::PackageList packages = selectedPackages();
@@ -562,9 +555,12 @@ void PackageWidget::setPackagesLocked(bool lock)
         return;
 
     for (QApt::Package *package : packages) {
-        bool locked = setLocked(package, lock);
-        if (!locked) {
-            // TODO: report error
+        if (!m_backend->setPackagePinned(package, lock)) {
+            QString title = i18nc("@title:window", "Failed to Lock Package");
+            QString text = i18nc("@info Error text", "The package %1 could not "
+                                 "be locked. Failed to write lock file.",
+                                 package->name());
+            KMessageBox::error(this, text, title);
         }
     }
 
@@ -612,7 +608,7 @@ void PackageWidget::showBrokenReason(QApt::Package *package)
     QHash<int, QHash<QString, QVariantMap> > failedReasons = package->brokenReason();
     QString reason;
     QString dialogText = i18nc("@label", "The \"%1\" package could not be marked for installation or upgrade:",
-                               package->latin1Name());
+                               package->name());
     dialogText += '\n';
     QString title = i18nc("@title:window", "Unable to Mark Package");
 
@@ -639,7 +635,7 @@ QString PackageWidget::digestReason(QApt::Package *pkg, QApt::BrokenReason failT
         reason = i18nc("@label", "The \"%1\" package has no available version, but exists in the database.\n"
                        "\tThis typically means that the package was mentioned in a dependency and "
                        "never uploaded, has been obsoleted, or is not available from the currently-enabled "
-                       "repositories.", pkg->latin1Name());
+                       "repositories.", pkg->name());
         break;
     }
     case QApt::WrongCandidateVersion: {
@@ -724,5 +720,3 @@ QString PackageWidget::digestReason(QApt::Package *pkg, QApt::BrokenReason failT
 
     return reason;
 }
-
-#include "PackageWidget.moc"
