@@ -59,7 +59,6 @@ public:
     };
 
     PackageWidget(QWidget *parent);
-    ~PackageWidget();
 
     void setPackagesType(int type);
     void setHeaderText(const QString &text);
@@ -132,12 +131,12 @@ private Q_SLOTS:
     void setPackagesPurge();
     void setKeep(QApt::Package *package);
     void setPackagesKeep();
-    bool setLocked(QApt::Package *package, bool lock);
     void setPackagesLocked(bool lock);
     void showBrokenReason(QApt::Package *package);
 
 Q_SIGNALS:
     void packageChanged();
+    void doneSortingPackages(bool sorting);
 };
 
 #endif
